/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry.core;

import java.net.URI;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.petals.registry.api.Information;
import org.ow2.petals.registry.api.Query;
import org.ow2.petals.registry.api.RemoteRegistry;
import org.ow2.petals.registry.api.Resource;
import org.ow2.petals.registry.api.context.Context;
import org.ow2.petals.registry.api.exception.LifeCycleException;
import org.ow2.petals.registry.api.exception.RegistryException;
import org.ow2.petals.registry.api.lifecycle.AbstractLifeCycle;
import org.ow2.petals.registry.api.lifecycle.LifeCycle;
import org.ow2.petals.registry.api.transport.MessageSender;

/**
 * A Remote Registry access implementation. 
 * <p>
 * This is the remote vision from the local container.
 * </p>
 * 
 * @author Christophe HAMERLING - EBM WebSourcing
 */
public class RemoteRegistryImpl implements RemoteRegistry {

    private final Log log = LogFactory.getLog(RemoteRegistryImpl.class);

    /**
     * The remote URI
     */
    private URI uri;

    /**
     * The remote name
     */
    private String name;

    /**
     * The message sender used to send messages to the remote registry
     */
    private MessageSender messageSender;

    /**
     * The lifecycle
     */
    private final LifeCycle lifeCycle;

    /**
     * Creates a new instance of RemoteRegistryImpl
     */
    public RemoteRegistryImpl() {
        this.lifeCycle = new AbstractLifeCycle() {
            @Override
            protected void doInit(Context context) throws LifeCycleException {
                RemoteRegistryImpl.this.log
                        .debug("FIXME : Check the foreign registry at initialization?");
            }
        };
    }

    public Resource get(final String path, final Information information) throws RegistryException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Get the resoucre on path " + path);
        }
        if (this.messageSender == null) {
            throw new RegistryException(
                    "No message sender found to send message to remote registry");
        }
        return this.messageSender.get(path, information);
    }
    
    public List<Resource> getAll(final String path, final Information information) throws RegistryException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Get all the resources under path " + path);
        }
        if (this.messageSender == null) {
            throw new RegistryException(
                    "No message sender found to send message to remote registry");
        }
        return this.messageSender.getAll(path, information);
    }

    public boolean put(final String path, final Resource resource, final Information information) throws RegistryException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Put the resource " + resource);
        }

        if (this.messageSender == null) {
            throw new RegistryException(
                    "No message sender found to send message to remote registry");
        }
        return this.messageSender.put(path, resource, information);
    }

    public boolean delete(String path, Information information) throws RegistryException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Delete the resource at " + path);
        }

        if (this.messageSender == null) {
            throw new RegistryException(
                    "No message sender found to send message to remote registry");
        }
        return this.messageSender.delete(path, information);
    }

    public List<Resource> query(Query query, Information info) throws RegistryException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Query resources with " + query);
        }

        if (this.messageSender == null) {
            throw new RegistryException(
                    "No message sender found to send message to remote registry");
        }
        return this.messageSender.query(query, info);
    }

    public URI getURI() {
        return this.uri;
    }

    public String getName() {
        return this.name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public STATE getState() {
        return this.lifeCycle.getState();
    }

    public void init(Context context) throws LifeCycleException {
        this.lifeCycle.init(context);
    }

    public boolean isInitialized() {
        return this.lifeCycle.isInitialized();
    }

    public boolean isNotInitialized() {
        return this.lifeCycle.isNotInitialized();
    }

    public boolean isRunning() {
        return this.lifeCycle.isRunning();
    }

    public boolean isStarted() {
        return this.lifeCycle.isStarted();
    }

    public boolean isStopped() {
        return this.lifeCycle.isStopped();
    }

    public void start() throws LifeCycleException {
        this.lifeCycle.start();
    }

    public void stop() throws LifeCycleException {
        this.lifeCycle.stop();
    }

    public void setURI(URI uri) {
        this.uri = uri;
    }

    public MessageSender getMessageSender() {
        return this.messageSender;
    }

    public void setMessageSender(MessageSender messageSender) {
        this.messageSender = messageSender;
        
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        builder.append("RemoteRegistryImpl [name=");
        builder.append(this.name);
        builder.append(", uri=");
        builder.append(this.uri);
        builder.append("]");
        return builder.toString();
    }
}
